@@ -28,7 +28,7 @@
 #include <string.h>
 
 static const char *device_validation_layers[] = {
-  "VK_LAYER_GOOGLE_threading",
+  "VK_LAYER_LUNARG_threading",
   "VK_LAYER_LUNARG_mem_tracker",
   "VK_LAYER_LUNARG_object_tracker",
   "VK_LAYER_LUNARG_draw_state",
@@ -42,6 +42,12 @@ static const char *device_validation_layers[] = {
 GST_DEBUG_CATEGORY (GST_CAT_DEFAULT);
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_CONTEXT);
 
+#define gst_vulkan_device_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstVulkanDevice, gst_vulkan_device, GST_TYPE_OBJECT,
+    GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "vulkandevice", 0,
+        "Vulkan Device");
+    GST_DEBUG_CATEGORY_GET (GST_CAT_CONTEXT, "GST_CONTEXT"));
+
 static void gst_vulkan_device_finalize (GObject * object);
 
 struct _GstVulkanDevicePrivate
@@ -49,20 +55,11 @@ struct _GstVulkanDevicePrivate
   gboolean opened;
 };
 
-#define gst_vulkan_device_parent_class parent_class
-G_DEFINE_TYPE_WITH_CODE (GstVulkanDevice, gst_vulkan_device, GST_TYPE_OBJECT,
-    G_ADD_PRIVATE (GstVulkanDevice)
-    GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "vulkandevice", 0,
-        "Vulkan Device");
-    GST_DEBUG_CATEGORY_GET (GST_CAT_CONTEXT, "GST_CONTEXT"));
-
 GstVulkanDevice *
 gst_vulkan_device_new (GstVulkanInstance * instance)
 {
   GstVulkanDevice *device = g_object_new (GST_TYPE_VULKAN_DEVICE, NULL);
 
-  gst_object_ref_sink (device);
-
   device->instance = gst_object_ref (instance);
   /* FIXME: select this externally */
   device->device_index = 0;
@@ -73,7 +70,8 @@ gst_vulkan_device_new (GstVulkanInstance * instance)
 static void
 gst_vulkan_device_init (GstVulkanDevice * device)
 {
-  device->priv = gst_vulkan_device_get_instance_private (device);
+  device->priv = G_TYPE_INSTANCE_GET_PRIVATE ((device),
+      GST_TYPE_VULKAN_DEVICE, GstVulkanDevicePrivate);
 }
 
 static void
@@ -81,6 +79,8 @@ gst_vulkan_device_class_init (GstVulkanDeviceClass * device_class)
 {
   GObjectClass *gobject_class = (GObjectClass *) device_class;
 
+  g_type_class_add_private (device_class, sizeof (GstVulkanDevicePrivate));
+
   gobject_class->finalize = gst_vulkan_device_finalize;
 }
 
@@ -143,14 +143,10 @@ _physical_device_info (GstVulkanDevice * device, GError ** error)
 
   vkGetPhysicalDeviceProperties (gpu, &props);
 
-  GST_INFO_OBJECT (device, "device name %s type %s api version %u.%u.%u, "
-      "driver version %u.%u.%u vendor ID 0x%x, device ID 0x%x",
-      props.deviceName, _device_type_to_string (props.deviceType),
-      VK_VERSION_MAJOR (props.apiVersion), VK_VERSION_MINOR (props.apiVersion),
-      VK_VERSION_PATCH (props.apiVersion),
-      VK_VERSION_MAJOR (props.driverVersion),
-      VK_VERSION_MINOR (props.driverVersion),
-      VK_VERSION_PATCH (props.driverVersion), props.vendorID, props.deviceID);
+  GST_INFO_OBJECT (device, "device name %s type %s api version %u, "
+      "driver version %u vendor ID 0x%x, device ID 0x%x", props.deviceName,
+      _device_type_to_string (props.deviceType), props.apiVersion,
+      props.driverVersion, props.vendorID, props.deviceID);
 
   return TRUE;
 }
@@ -230,7 +226,7 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
     goto error;
   }
 
-  for (i = 0; i < device_extension_count; i++) {
+  for (uint32_t i = 0; i < device_extension_count; i++) {
     GST_TRACE_OBJECT (device, "checking device extension %s",
         device_extensions[i].extensionName);
     if (!strcmp (VK_KHR_SWAPCHAIN_EXTENSION_NAME,
@@ -293,13 +289,8 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
     device_info.pNext = NULL;
     device_info.queueCreateInfoCount = 1;
     device_info.pQueueCreateInfos = &queue_info;
-#if 0
     device_info.enabledLayerCount = enabled_layer_count;
     device_info.ppEnabledLayerNames = (const char *const *) enabled_layers;
-#else
-    device_info.enabledLayerCount = 0;
-    device_info.ppEnabledLayerNames = NULL;
-#endif
     device_info.enabledExtensionCount = enabled_extension_count;
     device_info.ppEnabledExtensionNames = (const char *const *) extension_names;
     device_info.pEnabledFeatures = NULL;
@@ -350,7 +341,6 @@ gst_vulkan_device_get_queue (GstVulkanDevice * device, guint32 queue_family,
       device->queue_family_props[queue_family].queueCount, NULL);
 
   ret = g_object_new (GST_TYPE_VULKAN_QUEUE, NULL);
-  gst_object_ref_sink (ret);
   ret->device = gst_object_ref (device);
   ret->family = queue_family;
   ret->index = queue_i;
@@ -545,12 +535,12 @@ gst_vulkan_device_run_context_query (GstElement * element,
     gst_query_parse_context (query, &context);
     if (context)
       gst_context_get_vulkan_device (context, device);
-
-    gst_query_unref (query);
   }
 
   GST_DEBUG_OBJECT (element, "found device %p", *device);
 
+  gst_query_unref (query);
+
   if (*device)
     return TRUE;
 
