@@ -29,3 +29,5 @@
 #include "debug.h"
 #include "vao.h"
 #include "sync.h"
+#include "buffers.h"
+#include "query.h"
