@@ -91,11 +91,13 @@
 #include <gst/tag/tag.h>
 #include <gst/video/video.h>
 #include <gst/mpegts/mpegts.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "mpegtsmux.h"
 
 #include "mpegtsmux_aac.h"
 #include "mpegtsmux_ttxt.h"
+#include "mpegtsmux_opus.h"
 
 GST_DEBUG_CATEGORY (mpegtsmux_debug);
 #define GST_CAT_DEFAULT mpegtsmux_debug
@@ -125,12 +127,16 @@ static GstStaticPadTemplate mpegtsmux_sink_factory =
         "video/x-dirac;"
         "video/x-h264,stream-format=(string)byte-stream,"
         "alignment=(string){au, nal}; "
+        "video/x-h265,stream-format=(string)byte-stream,"
+        "alignment=(string){au, nal}; "
         "audio/mpeg, "
         "parsed = (boolean) TRUE, "
         "mpegversion = (int) { 1, 2 };"
         "audio/mpeg, "
         "framed = (boolean) TRUE, "
-        "mpegversion = (int) 4, stream-format = (string) { raw, adts };"
+        "mpegversion = (int) 4, stream-format = (string) adts;"
+        "audio/mpeg, "
+        "mpegversion = (int) 4, stream-format = (string) raw;"
         "audio/x-lpcm, "
         "width = (int) { 16, 20, 24 }, "
         "rate = (int) { 48000, 96000 }, "
@@ -140,7 +146,10 @@ static GstStaticPadTemplate mpegtsmux_sink_factory =
         "mute = (boolean) { FALSE, TRUE }; "
         "audio/x-ac3, framed = (boolean) TRUE;"
         "audio/x-dts, framed = (boolean) TRUE;"
-        "subpicture/x-dvb;" "application/x-teletext"));
+        "audio/x-opus, "
+        "channels = (int) [1, 8], "
+        "channel-mapping-family = (int) {0, 1};"
+        "subpicture/x-dvb; application/x-teletext; meta/x-klv, parsed=true"));
 
 static GstStaticPadTemplate mpegtsmux_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -330,8 +339,7 @@ mpegtsmux_init (MpegTsMux * mux)
 static void
 mpegtsmux_pad_reset (MpegTsPadData * pad_data)
 {
-  pad_data->pid = 0;
-  pad_data->min_dts = GST_CLOCK_TIME_NONE;
+  pad_data->dts = GST_CLOCK_STIME_NONE;
   pad_data->prog_id = -1;
 #if 0
   pad_data->prog_id = -1;
@@ -361,6 +369,7 @@ mpegtsmux_pad_reset (MpegTsPadData * pad_data)
 static void
 mpegtsmux_reset (MpegTsMux * mux, gboolean alloc)
 {
+  GstBuffer *buf;
   GSList *walk;
 
   mux->first = TRUE;
@@ -390,21 +399,14 @@ mpegtsmux_reset (MpegTsMux * mux, gboolean alloc)
     mux->tsmux = NULL;
   }
 
-  memset (mux->programs, 0, sizeof (mux->programs));
-
-  if (mux->streamheader) {
-    GstBuffer *buf;
-    GList *sh;
-
-    sh = mux->streamheader;
-    while (sh) {
-      buf = sh->data;
-      gst_buffer_unref (buf);
-      sh = g_list_next (sh);
-    }
-    g_list_free (mux->streamheader);
-    mux->streamheader = NULL;
+  if (mux->programs) {
+    g_hash_table_destroy (mux->programs);
   }
+  mux->programs = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+  while ((buf = g_queue_pop_head (&mux->streamheader)))
+    gst_buffer_unref (buf);
+
   gst_event_replace (&mux->force_key_unit_event, NULL);
   gst_buffer_replace (&mux->out_buffer, NULL);
 
@@ -445,6 +447,10 @@ mpegtsmux_dispose (GObject * object)
     gst_structure_free (mux->prog_map);
     mux->prog_map = NULL;
   }
+  if (mux->programs) {
+    g_hash_table_destroy (mux->programs);
+    mux->programs = NULL;
+  }
   GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
@@ -581,6 +587,7 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data)
   const gchar *mt;
   const GValue *value = NULL;
   GstBuffer *codec_data = NULL;
+  guint8 opus_channel_config_code = 0;
 
   pad = ts_data->collect.pad;
   caps = gst_pad_get_current_caps (pad);
@@ -607,6 +614,8 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data)
     st = TSMUX_ST_PS_AUDIO_LPCM;
   } else if (strcmp (mt, "video/x-h264") == 0) {
     st = TSMUX_ST_VIDEO_H264;
+  } else if (strcmp (mt, "video/x-h265") == 0) {
+    st = TSMUX_ST_VIDEO_HEVC;
   } else if (strcmp (mt, "audio/mpeg") == 0) {
     gint mpegversion;
 
@@ -668,6 +677,68 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data)
     st = TSMUX_ST_PS_TELETEXT;
     /* needs a particularly sized layout */
     ts_data->prepare_func = mpegtsmux_prepare_teletext;
+  } else if (strcmp (mt, "audio/x-opus") == 0) {
+    guint8 channels, mapping_family, stream_count, coupled_count;
+    guint8 channel_mapping[256];
+
+    if (!gst_codec_utils_opus_parse_caps (caps, NULL, &channels,
+            &mapping_family, &stream_count, &coupled_count, channel_mapping)) {
+      GST_ERROR_OBJECT (pad, "Incomplete Opus caps");
+      goto not_negotiated;
+    }
+
+    if (channels <= 2 && mapping_family == 0) {
+      opus_channel_config_code = channels;
+    } else if (channels == 2 && mapping_family == 255 && stream_count == 1
+        && coupled_count == 1) {
+      /* Dual mono */
+      opus_channel_config_code = 0;
+    } else if (channels >= 2 && channels <= 8 && mapping_family == 1) {
+      static const guint8 coupled_stream_counts[9] = {
+        1, 0, 1, 1, 2, 2, 2, 3, 3
+      };
+      static const guint8 channel_map_a[8][8] = {
+        {0},
+        {0, 1},
+        {0, 2, 1},
+        {0, 1, 2, 3},
+        {0, 4, 1, 2, 3},
+        {0, 4, 1, 2, 3, 5},
+        {0, 4, 1, 2, 3, 5, 6},
+        {0, 6, 1, 2, 3, 4, 5, 7},
+      };
+      static const guint8 channel_map_b[8][8] = {
+        {0},
+        {0, 1},
+        {0, 1, 2},
+        {0, 1, 2, 3},
+        {0, 1, 2, 3, 4},
+        {0, 1, 2, 3, 4, 5},
+        {0, 1, 2, 3, 4, 5, 6},
+        {0, 1, 2, 3, 4, 5, 6, 7},
+      };
+
+      /* Vorbis mapping */
+      if (stream_count == channels - coupled_stream_counts[channels] &&
+          coupled_count == coupled_stream_counts[channels] &&
+          memcmp (channel_mapping, channel_map_a[channels - 1],
+              channels) == 0) {
+        opus_channel_config_code = channels;
+      } else if (stream_count == channels - coupled_stream_counts[channels] &&
+          coupled_count == coupled_stream_counts[channels] &&
+          memcmp (channel_mapping, channel_map_b[channels - 1],
+              channels) == 0) {
+        opus_channel_config_code = channels | 0x80;
+      } else {
+        GST_FIXME_OBJECT (pad, "Opus channel mapping not handled");
+        goto not_negotiated;
+      }
+    }
+
+    st = TSMUX_ST_PS_OPUS;
+    ts_data->prepare_func = mpegtsmux_prepare_opus;
+  } else if (strcmp (mt, "meta/x-klv") == 0) {
+    st = TSMUX_ST_PS_KLV;
   }
 
   if (st != TSMUX_ST_RESERVED) {
@@ -682,6 +753,8 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data)
     gst_structure_get_int (s, "channels", &ts_data->stream->audio_channels);
     gst_structure_get_int (s, "bitrate", &ts_data->stream->audio_bitrate);
 
+    ts_data->stream->opus_channel_config_code = opus_channel_config_code;
+
     tsmux_stream_set_buffer_release_func (ts_data->stream, release_buffer_cb);
     tsmux_program_add_stream (ts_data->prog, ts_data->stream);
 
@@ -750,10 +823,10 @@ mpegtsmux_create_streams (MpegTsMux * mux)
               ("Reading program map failed. Assuming default"), (NULL));
           idx = DEFAULT_PROG_ID;
         }
-        if (idx < 0 || idx >= MAX_PROG_NUMBER) {
-          GST_DEBUG_OBJECT (mux, "Program number %d associate with pad %s out "
-              "of range (max = %d); DEFAULT_PROGRAM = %d is used instead",
-              idx, name, MAX_PROG_NUMBER, DEFAULT_PROG_ID);
+        if (idx < 0) {
+          GST_DEBUG_OBJECT (mux, "Program number %d associate with pad %s less "
+              "than zero; DEFAULT_PROGRAM = %d is used instead",
+              idx, name, DEFAULT_PROG_ID);
           idx = DEFAULT_PROG_ID;
         }
         ts_data->prog_id = idx;
@@ -762,13 +835,15 @@ mpegtsmux_create_streams (MpegTsMux * mux)
       }
     }
 
-    ts_data->prog = mux->programs[ts_data->prog_id];
+    ts_data->prog =
+        g_hash_table_lookup (mux->programs, GINT_TO_POINTER (ts_data->prog_id));
     if (ts_data->prog == NULL) {
       ts_data->prog = tsmux_program_new (mux->tsmux, ts_data->prog_id);
       if (ts_data->prog == NULL)
         goto no_program;
       tsmux_set_pmt_interval (ts_data->prog, mux->pmt_interval);
-      mux->programs[ts_data->prog_id] = ts_data->prog;
+      g_hash_table_insert (mux->programs,
+          GINT_TO_POINTER (ts_data->prog_id), ts_data->prog);
     }
 
     if (ts_data->stream == NULL) {
@@ -858,6 +933,10 @@ mpegtsmux_sink_event (GstCollectPads * pads, GstCollectData * data,
 
         lang_code = gst_tag_get_language_code_iso_639_2B (lang);
         if (lang_code) {
+          if (pad_data->language) {
+            g_free (pad_data->language);
+            pad_data->language = NULL;
+          }
           GST_DEBUG_OBJECT (pad, "Setting language to '%s'", lang_code);
           pad_data->language = g_strdup (lang_code);
         } else {
@@ -871,6 +950,19 @@ mpegtsmux_sink_event (GstCollectPads * pads, GstCollectData * data,
       forward = gst_tag_list_get_scope (list) == GST_TAG_SCOPE_GLOBAL;
       break;
     }
+    case GST_EVENT_STREAM_START:{
+      GstStreamFlags flags;
+
+      gst_event_parse_stream_flags (event, &flags);
+
+      /* Don't wait for data on sparse inputs like metadata streams */
+      if ((flags & GST_STREAM_FLAG_SPARSE)) {
+        GST_COLLECT_PADS_STATE_UNSET (data, GST_COLLECT_PADS_STATE_LOCKED);
+        gst_collect_pads_set_waiting (pads, data, FALSE);
+        GST_COLLECT_PADS_STATE_SET (data, GST_COLLECT_PADS_STATE_LOCKED);
+      }
+      break;
+    }
     default:
       break;
   }
@@ -981,8 +1073,7 @@ check_pending_key_unit_event (GstEvent * pending_event, GstSegment * segment,
       timestamp == GST_CLOCK_TIME_NONE)
     goto out;
 
-  running_time = gst_segment_to_running_time (segment,
-      GST_FORMAT_TIME, timestamp);
+  running_time = timestamp;
 
   GST_INFO ("now %" GST_TIME_FORMAT " wanted %" GST_TIME_FORMAT,
       GST_TIME_ARGS (running_time), GST_TIME_ARGS (pending_key_unit_ts));
@@ -1026,7 +1117,7 @@ mpegtsmux_clip_inc_running_time (GstCollectPads * pads,
   *outbuf = buf;
 
   /* PTS */
-  time = GST_BUFFER_TIMESTAMP (buf);
+  time = GST_BUFFER_PTS (buf);
 
   /* invalid left alone and passed */
   if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (time))) {
@@ -1037,11 +1128,11 @@ mpegtsmux_clip_inc_running_time (GstCollectPads * pads,
       *outbuf = NULL;
       goto beach;
     } else {
-      GST_LOG_OBJECT (cdata->pad, "buffer pts %" GST_TIME_FORMAT " -> %"
+      GST_LOG_OBJECT (cdata->pad, "buffer pts %" GST_TIME_FORMAT " ->  %"
           GST_TIME_FORMAT " running time",
-          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)), GST_TIME_ARGS (time));
+          GST_TIME_ARGS (GST_BUFFER_PTS (buf)), GST_TIME_ARGS (time));
       buf = *outbuf = gst_buffer_make_writable (buf);
-      GST_BUFFER_TIMESTAMP (*outbuf) = time;
+      GST_BUFFER_PTS (*outbuf) = time;
     }
   }
 
@@ -1050,32 +1141,36 @@ mpegtsmux_clip_inc_running_time (GstCollectPads * pads,
 
   /* invalid left alone and passed */
   if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (time))) {
-    time = gst_segment_to_running_time (&cdata->segment, GST_FORMAT_TIME, time);
-    /* may have to decode out-of-segment, so pass INVALID */
-    if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (time))) {
-      GST_DEBUG_OBJECT (cdata->pad, "running dts outside segment");
-    } else {
-      GST_LOG_OBJECT (cdata->pad, "buffer dts %" GST_TIME_FORMAT " -> %"
-          GST_TIME_FORMAT " running time",
-          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)), GST_TIME_ARGS (time));
-      if (GST_CLOCK_TIME_IS_VALID (pad_data->min_dts) &&
-          time < pad_data->min_dts) {
-        /* Ignore DTS going backward */
-        GST_WARNING_OBJECT (cdata->pad, "ignoring DTS going backward");
-        time = pad_data->min_dts;
-      }
-      *outbuf = gst_buffer_make_writable (buf);
-      GST_BUFFER_DTS (*outbuf) = time;
+    gint sign;
+    gint64 dts;
+
+    sign = gst_segment_to_running_time_full (&cdata->segment, GST_FORMAT_TIME,
+        time, &time);
+
+    if (sign > 0)
+      dts = (gint64) time;
+    else
+      dts = -((gint64) time);
+
+    GST_LOG_OBJECT (cdata->pad, "buffer dts %" GST_TIME_FORMAT " -> %"
+        GST_STIME_FORMAT " running time", GST_TIME_ARGS (GST_BUFFER_DTS (buf)),
+        GST_STIME_ARGS (dts));
+
+    if (GST_CLOCK_STIME_IS_VALID (pad_data->dts) && dts < pad_data->dts) {
+      /* Ignore DTS going backward */
+      GST_WARNING_OBJECT (cdata->pad, "ignoring DTS going backward");
+      dts = pad_data->dts;
     }
-  }
 
-  buf = *outbuf;
-  if (pad_data->prepare_func) {
-    MpegTsMux *mux = (MpegTsMux *) user_data;
+    *outbuf = gst_buffer_make_writable (buf);
+    if (sign > 0)
+      GST_BUFFER_DTS (*outbuf) = time;
+    else
+      GST_BUFFER_DTS (*outbuf) = GST_CLOCK_TIME_NONE;
 
-    *outbuf = pad_data->prepare_func (buf, pad_data, mux);
-    g_assert (*outbuf);
-    gst_buffer_unref (buf);
+    pad_data->dts = dts;
+  } else {
+    pad_data->dts = GST_CLOCK_STIME_NONE;
   }
 
 beach:
@@ -1089,8 +1184,8 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
   GstFlowReturn ret = GST_FLOW_OK;
   MpegTsPadData *best = (MpegTsPadData *) data;
   TsMuxProgram *prog;
-  gint64 pts = -1;
-  guint64 dts = -1;
+  gint64 pts = GST_CLOCK_STIME_NONE;
+  gint64 dts = GST_CLOCK_STIME_NONE;
   gboolean delta = TRUE, header = FALSE;
   StreamData *stream_data;
 
@@ -1098,8 +1193,11 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
 
   if (G_UNLIKELY (mux->first)) {
     ret = mpegtsmux_create_streams (mux);
-    if (G_UNLIKELY (ret != GST_FLOW_OK))
+    if (G_UNLIKELY (ret != GST_FLOW_OK)) {
+      if (buf)
+        gst_buffer_unref (buf);
       return ret;
+    }
 
     mpegtsmux_prepare_srcpad (mux);
 
@@ -1108,11 +1206,15 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
 
   if (G_UNLIKELY (best == NULL)) {
     /* EOS */
+    GST_INFO_OBJECT (mux, "EOS");
     /* drain some possibly cached data */
     new_packet_m2ts (mux, NULL, -1);
     mpegtsmux_push_packets (mux, TRUE);
     gst_pad_push_event (mux->srcpad, gst_event_new_eos ());
 
+    if (buf)
+      gst_buffer_unref (buf);
+
     return GST_FLOW_OK;
   }
 
@@ -1122,11 +1224,20 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
 
   g_assert (buf != NULL);
 
+  if (best->prepare_func) {
+    GstBuffer *tmp;
+
+    tmp = best->prepare_func (buf, best, mux);
+    g_assert (tmp);
+    gst_buffer_unref (buf);
+    buf = tmp;
+  }
+
   if (mux->force_key_unit_event != NULL && best->stream->is_video_stream) {
     GstEvent *event;
 
     event = check_pending_key_unit_event (mux->force_key_unit_event,
-        &best->collect.segment, GST_BUFFER_TIMESTAMP (buf),
+        &best->collect.segment, GST_BUFFER_PTS (buf),
         GST_BUFFER_FLAGS (buf), mux->pending_key_unit_ts);
     if (event) {
       GstClockTime running_time;
@@ -1161,7 +1272,7 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
     /* Take the first data stream for the PCR */
     GST_DEBUG_OBJECT (COLLECT_DATA_PAD (best),
         "Use stream (pid=%d) from pad as PCR for program (prog_id = %d)",
-        MPEG_TS_PAD_DATA (best)->pid, MPEG_TS_PAD_DATA (best)->prog_id);
+        best->pid, best->prog_id);
 
     /* Set the chosen PCR stream */
     tsmux_program_set_pcr_stream (prog, best->stream);
@@ -1172,18 +1283,18 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
 
   if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_PTS (buf))) {
     pts = GSTTIME_TO_MPEGTIME (GST_BUFFER_PTS (buf));
-    GST_DEBUG_OBJECT (mux, "Buffer has PTS %" GST_TIME_FORMAT " pts %"
+    GST_DEBUG_OBJECT (mux, "Buffer has PTS  %" GST_TIME_FORMAT " pts %"
         G_GINT64_FORMAT, GST_TIME_ARGS (GST_BUFFER_PTS (buf)), pts);
   }
 
-  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (buf))) {
-    dts = GSTTIME_TO_MPEGTIME (GST_BUFFER_DTS (buf));
-    GST_DEBUG_OBJECT (mux, "Buffer has DTS %" GST_TIME_FORMAT " dts %"
-        G_GINT64_FORMAT, GST_TIME_ARGS (GST_BUFFER_DTS (buf)), dts);
+  if (GST_CLOCK_STIME_IS_VALID (best->dts)) {
+    dts = GSTTIME_TO_MPEGTIME (best->dts);
+    GST_DEBUG_OBJECT (mux, "Buffer has DTS %" GST_STIME_FORMAT " dts %"
+        G_GINT64_FORMAT, GST_STIME_ARGS (best->dts), dts);
   }
 
   /* should not have a DTS without PTS */
-  if (pts == -1 && dts != -1) {
+  if (!GST_CLOCK_STIME_IS_VALID (pts) && GST_CLOCK_STIME_IS_VALID (dts)) {
     GST_DEBUG_OBJECT (mux, "using DTS for unknown PTS");
     pts = dts;
   }
@@ -1202,6 +1313,14 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
     GST_OBJECT_UNLOCK (mux);
 #endif
   }
+
+  if (best->stream->is_meta && gst_buffer_get_size (buf) > (G_MAXUINT16 - 3)) {
+    GST_WARNING_OBJECT (mux, "KLV meta unit too big, splitting not supported");
+
+    gst_buffer_unref (buf);
+    return GST_FLOW_OK;
+  }
+
   GST_DEBUG_OBJECT (mux, "delta: %d", delta);
 
   stream_data = stream_data_new (buf);
@@ -1238,6 +1357,8 @@ write_fail:
   }
 no_program:
   {
+    if (buf)
+      gst_buffer_unref (buf);
     GST_ELEMENT_ERROR (mux, STREAM, MUX,
         ("Stream on pad %" GST_PTR_FORMAT
             " is not associated with any program", COLLECT_DATA_PAD (best)),
@@ -1339,8 +1460,11 @@ new_packet_common_init (MpegTsMux * mux, GstBuffer * buf, guint8 * data,
       } else {
         hbuf = gst_buffer_copy (buf);
       }
-      mux->streamheader = g_list_append (mux->streamheader, hbuf);
-    } else if (mux->streamheader) {
+      GST_LOG_OBJECT (mux,
+          "Collecting packet with pid 0x%04x into streamheaders", pid);
+
+      g_queue_push_tail (&mux->streamheader, hbuf);
+    } else if (!g_queue_is_empty (&mux->streamheader)) {
       mpegtsmux_set_header_on_caps (mux);
       mux->streamheader_sent = TRUE;
     }
@@ -1641,26 +1765,22 @@ mpegtsmux_set_header_on_caps (MpegTsMux * mux)
   GValue array = { 0 };
   GValue value = { 0 };
   GstCaps *caps;
-  GList *sh;
 
   caps = gst_caps_make_writable (gst_pad_get_current_caps (mux->srcpad));
   structure = gst_caps_get_structure (caps, 0);
 
   g_value_init (&array, GST_TYPE_ARRAY);
 
-  sh = mux->streamheader;
-  while (sh) {
-    buf = sh->data;
+  GST_LOG_OBJECT (mux, "setting %u packets into streamheader",
+      g_queue_get_length (&mux->streamheader));
+
+  while ((buf = g_queue_pop_head (&mux->streamheader))) {
     g_value_init (&value, GST_TYPE_BUFFER);
     gst_value_take_buffer (&value, buf);
     gst_value_array_append_value (&array, &value);
     g_value_unset (&value);
-    sh = g_list_next (sh);
   }
 
-  g_list_free (mux->streamheader);
-  mux->streamheader = NULL;
-
   gst_structure_set_value (structure, "streamheader", &array);
   gst_pad_set_caps (mux->srcpad, caps);
   g_value_unset (&array);
